"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_template_compiler_1 = require("vue-template-compiler");
const nested_map_1 = require("./nested-map");
const routeMetaName = 'route-meta';
function resolveRoutePaths(paths, importPrefix, readFile) {
    const map = {};
    const splitted = paths.map(p => p.split('/'));
    splitted.forEach(path => {
        nested_map_1.setToMap(map, pathToMapPath(path), path);
    });
    return pathMapToMeta(map, importPrefix, readFile);
}
exports.resolveRoutePaths = resolveRoutePaths;
function pathMapToMeta(map, importPrefix, readFile, parentDepth = 0) {
    if (map.value) {
        const path = map.value;
        const meta = {
            name: pathToName(path),
            specifier: pathToSpecifier(path),
            path: pathToRoute(path, parentDepth),
            pathSegments: toActualPath(path),
            component: importPrefix + path.join('/')
        };
        const content = readFile(path.join('/'));
        const parsed = vue_template_compiler_1.parseComponent(content);
        const routeMetaBlock = parsed.customBlocks.find(b => b.type === routeMetaName);
        if (routeMetaBlock) {
            try {
                meta.routeMeta = JSON.parse(routeMetaBlock.content);
            }
            catch (err) {
                throw new Error(`Invalid json format of <route-meta> content in ${path.join('/')}\n` +
                    err.message);
            }
        }
        if (map.children) {
            meta.children = pathMapChildrenToMeta(map.children, importPrefix, readFile, path.length);
        }
        return [meta];
    }
    return map.children
        ? pathMapChildrenToMeta(map.children, importPrefix, readFile, parentDepth)
        : [];
}
function routePathComparator(a, b) {
    const a0 = a[0];
    const b0 = b[0];
    if (!a0 || !b0) {
        return 0;
    }
    const aOrder = isDynamicRoute(a0) ? 1 : 0;
    const bOrder = isDynamicRoute(b0) ? 1 : 0;
    const order = aOrder - bOrder;
    return order !== 0 ? order : routePathComparator(a.slice(1), b.slice(1));
}
function pathMapChildrenToMeta(children, importPrefix, readFile, parentDepth) {
    return Array.from(children.values())
        .reduce((acc, value) => {
        return acc.concat(pathMapToMeta(value, importPrefix, readFile, parentDepth));
    }, [])
        .sort((a, b) => {
        // Prioritize static routes than dynamic routes
        return routePathComparator(a.pathSegments, b.pathSegments);
    });
}
function isDynamicRoute(segment) {
    return segment[0] === ':';
}
function isOmittable(segment) {
    return segment === 'index.vue';
}
/**
 * - Remove `.vue` from the last path
 * - Omit if the last segument is `index`
 * - Convert dynamic route to `:param` format
 */
function toActualPath(segments) {
    const last = segments[segments.length - 1];
    if (isOmittable(last)) {
        segments = segments.slice(0, -1);
    }
    else {
        segments = segments.slice(0, -1).concat(basename(last));
    }
    return segments.map(s => {
        return s[0] === '_' ? ':' + s.slice(1) : s;
    });
}
function pathToMapPath(segments) {
    const last = segments[segments.length - 1];
    return segments.slice(0, -1).concat(basename(last));
}
function pathToName(segments) {
    const last = segments[segments.length - 1];
    segments = segments.slice(0, -1).concat(basename(last));
    return segments
        .map(s => {
        return s[0] === '_' ? s.slice(1) : s;
    })
        .join('-');
}
function pathToSpecifier(segments) {
    const name = pathToName(segments);
    const replaced = name
        .replace(/(^|[^a-zA-Z])([a-zA-Z])/g, (_, a, b) => {
        return a + b.toUpperCase();
    })
        .replace(/[^a-zA-Z0-9]/g, '');
    return /^\d/.test(replaced) ? '_' + replaced : replaced;
}
function pathToRoute(segments, parentDepth) {
    const prefix = parentDepth > 0 ? '' : '/';
    return (prefix +
        toActualPath(segments)
            .slice(parentDepth)
            .join('/'));
}
function basename(filename) {
    return filename.replace(/\..+?$/, '');
}
